﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace ParticleSim
{
    class Simulator : Microsoft.Xna.Framework.Game
    {
        public bool fullscreen = false;
        public bool useDefaultResolution = false;
        public static int width = 1920;
        public static int height = 1080;
        public int numberOfParticles = 20;
        public static GraphicsDeviceManager graphics;
        private SpriteBatch batch;
        public static Texture2D texture;
        public static List<Particle> particles = new List<Particle>();
        public static Random random = new Random();


        // Constructor
        public Simulator() : base()
        {
            // Set Display Properties
            graphics = new GraphicsDeviceManager(this);
            graphics.IsFullScreen = fullscreen;
            DisplayMode displayMode = GraphicsAdapter.DefaultAdapter.CurrentDisplayMode;

            if (useDefaultResolution)
            {
                width = displayMode.Width;
                height = displayMode.Height;
            }

            graphics.PreferredBackBufferWidth = width;
            graphics.PreferredBackBufferHeight = height;
            graphics.SynchronizeWithVerticalRetrace = true;
            graphics.PreferMultiSampling = true;
            graphics.HardwareModeSwitch = true;
            IsMouseVisible = true;
        }

        // Load Content
        protected override void LoadContent()
        {
            batch = new SpriteBatch(GraphicsDevice);
            FileStream textureStream = File.Open(@"tex/tex.png", FileMode.Open);
            texture = Texture2D.FromStream(Program.simulator.GraphicsDevice, textureStream);
            textureStream.Dispose();

            // Set texture to solid white color
            //texture = new Texture2D(GraphicsDevice, 128, 128);
            //texture.SetData(new[] { Color.White });

            /*Color[] colors = new Color[16384];
            for (int i = 0; i < 16384; i++)
            {
                colors[i].R = 255;
                colors[i].G = 255;
                colors[i].B = 255;
                colors[i].A = 255;
            }
            texture.SetData<Color>(colors);*/

            // Create particles
            for (int i = 0; i < numberOfParticles; i++)
            {
                Particle particle = new Particle();
                particle.transform.SetScaleRandom(3);
                //particle.sprite.SetColorRandom();
                particle.collider.SetVelocityRandom(5);
                particle.collider.mass = 0;
                particles.Add(particle);
            }
        }

        // Update
        protected override void Update(GameTime gameTime)
        {
            for (int i = 0; i < particles.Count; i++)
            {
                particles[i].Update();
            }

            if (Keyboard.GetState().IsKeyDown(Keys.Escape))
            {
                Program.simulator.Exit();
            }

            if (Keyboard.GetState().IsKeyDown(Keys.Space))
            {
                for (int i = 0; i < particles.Count; i++)
                {
                    particles[i].collider.SetVelocityRandom(30);
                }
            }
        }

        // Draw
        protected override void Draw(GameTime gameTime)
        {
            // Background Clear Color
            GraphicsDevice.Clear(Color.Black);

            // Sprite Batch
            batch.Begin(SpriteSortMode.FrontToBack, BlendState.AlphaBlend, SamplerState.PointClamp, DepthStencilState.None, RasterizerState.CullCounterClockwise);

            for (int i = 0; i < particles.Count; i++)
            {
                batch.Draw(particles[i].sprite.texture, particles[i].transform.position, particles[i].sprite.source, particles[i].sprite.color, 
                    particles[i].transform.rotation, particles[i].transform.origin, particles[i].transform.scale, SpriteEffects.None, 0);
            }
        
            batch.End();
        }
    }
}
